@@ -285,8 +285,9 @@ fmri.lm <- function(data,z,actype="accalc",hmax=3.52,vtype="var",step=0.01,contr
      scale <- max(abs(qscale))/32767
      residuals <- writeBin(as.integer(residuals/scale),raw(),2)
   }
-  bw <- optim(c(2,2,2),corrrisk,method="L-BFGS-B",lower=c(.25,.25,.25),lag=lags,data=corr)$par  
+  bw <- optim(c(2,2,2),corrrisk,method="L-BFGS-B",lower=c(.25,.25,.25),upper=c(4,4,4),lag=lags,data=corr)$par  
   bw[bw<=.25] <- 0
+  if( (max(bw) > 2.5 ) || (corr[lags[1],1,1]+corr[1,lags[2],1]+corr[1,1,lags[3]] >0.5) ) warning(paste("Local smoothness characterized by large bandwidth ",bw," check residuals for structure",collapse=","))
   rxyz <- c(resel(1,bw[1]), resel(1,bw[2]), resel(1,bw[3]))
   dim(rxyz) <- c(1,3)
 
